Option Explicit
Dim Context : Set Context = New clContext ' This line for running outside of WUG
'***Configuration
Const sGroupName = "Group Name" 'Name of the WhatsUp Gold Device Group to add devices to
Const sSNMPCred = "DD Meraki" 'Name of the credential inside WhatsUp Gold to associate with the devices. This should be your Meraki SNMP credential
Const sActionPolicy = "Sample Action Policy" 'Name of the action policy to apply to the added devices
Const sPathToFile = "C:\wugscripts\meraki_devices.csv" 'Path to the CSV file generated with Meraki_Gather_Devices script
Const sActiveMonitorName = "Meraki Status" 'Name of the active monitor to look at Meraki Status
Const sPerfMonName = "Meraki Utilization" 'Name of the performance monitor to gather Meraki interface utilization/traffic
Dim sDeviceTypeSwitch : sDeviceTypeSwitch = "Switch"
Dim sDeviceTypeAP : sDeviceTypeAP = "Wireless Access Point"
'***End Configuration

'IP Address of snmp.meraki.com
Dim sIPAddress : sIPAddress = DNSLookup("snmp.meraki.com")
'Create DB object
Dim oDb : Set oDb = Context.GetDB
'Create WUG Event Helper Object
Dim oEvent : Set oEvent = CreateObject("CoreAsp.EventHelper")

'Open and parse the CSV
Dim oTextFile, oFS, arrStr
Dim nCount : nCount = 0
Dim Name, NetworkName, Instance, Status, PublicIP, Serial, ProductCode, ProductDesc
Dim sDeviceType
set oFS = CreateObject("Scripting.FileSystemObject")
Set oTextFile = oFS.OpenTextFile(sPathToFile)
Do while NOT oTextFile.AtEndOfStream
 arrStr = split(oTextFile.ReadLine,",")
 Name = arrStr(0)
 NetworkName = arrStr(1)
 Instance = arrStr(2)
 Status = arrStr(3)
 PublicIP = arrStr(4)
 Serial = arrStr(5)
 ProductCode = arrStr(6)
 ProductDesc = arrStr(7)
 If InStr(1, ProductDesc, "AP", 1) <> 0 Then
  sDeviceType = sDeviceTypeAP
 Else
  sDeviceType = sDeviceTypeSwitch
 End If
 If nCount > 0 Then
  CreateDevice Name, NetworkName, Instance, Status, PublicIP, Serial, ProductCode, ProductDesc
 End If
 nCount = nCount + 1
 Loop
oTextFile.Close
Set oTextFile = Nothing
Set oFS = Nothing

' ****************
' * CreateDevice *
' ****************
Function CreateDevice(Name, NetworkName, Instance, Status, PublicIP, Serial, ProductCode, ProductDesc)
CreateDevice = 0
Dim sSql, sRS, bContinue
bContinue = True

Dim nDevTypeID, sSnmpOID, bSnmp
Dim nDevID, nCredID, nPolID, nActionPolicyID

Dim nCurrDev : nCurrDev = GetDevByName(Name)

If nCurrDev = 0 Then
 nDevTypeID = GetDevType(sDeviceType)
 If nDevTypeID = 0 Then
  Context.LogMessage "Invalid device type" & sDeviceType & ", will try using 'Device'..."
  nDevTypeID = GetDevType("Device")
 End If

If nDevTypeID = 0 Then
 Context.LogMessage "Unable to fine device type 'Device'"
 bContinue = False
End If

sSnmpOID = Instance
bSnmp = 1

nActionPolicyID = GetActionPolicyID(sActionPolicy)
If nActionPolicyID = "0" Then
   nActionPolicyID = Null
Else
   nActionPolicyID = CDbl(nActionPolicyID)
End If

' * Device *
If bContinue = True Then
   sSql = "Insert into Device (sDisplayName, nDeviceTypeID, bSnmpManageable, sSnmpOID, bAssumedState," &_
    "nWorstStateID, nBestStateID, nPollInterval, nDefaultNetworkInterfaceID, sNote, sStatus, sL2MainIPAddress, nActionPolicyID," &_
    "bGatherPerformanceData, bFireActions, bRemoved, sMaintenanceSchedule, bManualMaintenanceMode, nUnAcknowledgedActiveMonitors,"  &_
    "nUnAcknowledgedPassiveMonitors, bPollingOrder)" &_
	" Values (" & StringIfy(Name) & "," & NumIfy(nDevTypeID) & "," & NumIfy(bSnmp)& "," &_
    StringIfy(sSnmpOID) & ",0,0,0," & StringIfy("60") & ",-1,"& _
    StringIfy("Name:" & Name & vbCrLf & "Network Name:" & NetworkName & vbCrLf & "Instance:" & Instance) & ",'',NULL," & NumIfy(nActionPolicyID) & "," &_
	"0,0,0,NULL,0,0,0,NULL)"
	Context.LogMessage sSql
   Set sRS = oDb.Execute(sSql)
End If

' * Get DeviceID *
If bContinue = True Then
 sSql = "Select max(nDeviceID) as DevID from Device"
 Set sRS = oDb.Execute(sSql)
 nDevID = CDbl(sRS("DevID"))
 CreateDevice = CStr(nDevID)
End If

' * NetworkInterface *
If bContinue = True Then
 sSql = "Insert into NetworkInterface (nDeviceID, nPhysicalInterfaceID, nAddressType, bPollUsingNetworkName, sNetworkAddress, sNetworkName) " &_
  "Values (" & nDevID & ",NULL, 1, 0," & StringIfy (sIPAddress) & "," & StringIfy ("snmp.meraki.com") & ")"
 Set sRS = oDb.Execute(sSql)
End If

' * Update device with NetworkInterface *
If bContinue = True Then
   sSql = "Update device set nDefaultNetworkInterfaceID = " &_
  "(select Max(nNetworkInterfaceID) " &_
  "   from NetworkInterface) " &_
  " Where nDeviceID = " & nDevID
  Set sRS = oDb.Execute(sSql)
End If

' *Set to poll using hostname if necessary*
sSql = "update NetworkInterface set bPollUsingNetworkName = 1 where nDeviceID = " & nDevID
Set sRS = oDb.Execute(sSql)

'Get group ID
Dim nGroupID : nGroupID = GetCreateGroup(sGroupName)
If nGroupID = 0 Then
 Context.LogMessage "Invalid Group Name"
 bContinue = False
End If

' * Group assignment *
If bContinue = True Then
   sSql = "Insert into PivotDeviceToGroup (nDeviceID, nDeviceGroupID) " &_
          " Values (" & nDevID & "," & nGroupID & ")"
   Set sRS = oDb.Execute(sSql)
End If

' * SNMP Credentials *
If bContinue = True Then
   sSql = "Select max(nCredentialTypeID) AS nID from CredentialType " &_
          " Where nCredentialType in (1,2,4) " &_
          " and sDisplayName = " & StringIfy(sSNMPCred)
   Set sRS = oDb.Execute(sSql)
   nCredID = 0
   nCredID = sRS("nID")
   If nCredID > 0 Then
      Context.LogMessage "   Assigning SNMP credential id " & nCredID & " for " & sSNMPCred
      sSql = "Insert into PivotCredentialTypeToDevice (nCredentialTypeID, nDeviceID) " &_
             " Values (" & nCredID & "," & nDevID & ")"
      Set sRS = oDb.Execute(sSql)
   Else
      Context.LogMessage "   Invalid or no SNMP Credentials specified...skipping"
   End If
End If

' * Active Monitors *
Dim sMonName : sMonName = sActiveMonitorName
Dim sMonID
Dim sMonIntID, sMonInt, sMonActID, sMonAct
If Len(sMonName) > 0 Then
 sMonID = GetActMonByName(sMonName)
 If sMonID <> "0" Then
  Context.LogMessage "   Adding active monitor " & sMonName
  sSql = "Insert into PivotActiveMonitorTypeToDevice (nDeviceID, nActiveMonitorTypeId, bAssumedState, nMonitorStateID, bFireActions, bDisabled, bRemoved) " &_
   "Values (" & nDevID & "," & sMonID & ",0,0,0,0,0)"
  Set sRS = oDb.Execute(sSql)
  If Len(sMonInt) > 0 Then 
   sMonIntID = GetNetIntByString(nDevID, sMonInt)
   If sMonIntID <> "0" Then
    sSql = "update PivotActiveMonitorTypeToDevice set nNetworkInterfaceID = " & sMonIntID & " where nDeviceID = "& nDevID & " and nActiveMonitorTypeID = " & sMonID
    Set sRS = oDb.Execute(sSql)
   Else
    Context.LogMessage "   Could not find interfaceID for IP address " & sMonInt & "...will use default"
   End If
  End If        
  If Len(sMonAct) > 0 Then
   sMonActID = GetActionPolicyID(sMonAct)
   If sMonActID <> "0" Then
    sSql = "update PivotActiveMonitorTypeToDevice set nActionPolicyID = " & sMonActID & " where nDeviceID = "& nDevID & " and nActiveMonitorTypeID = " & sMonID
    Set sRS = oDb.Execute(sSql)
   Else
    Context.LogMessage "   Could not find ActionPolicyID for " & sMonAct & "...skipping"
   End If
  End If
 Else
  Context.LogMessage "   Oops, can't find active monitor " & sMonName
 End If
End If


' * Active Monitors *
sMonName = sPerfMonName
If Len(sMonName) > 0 Then
 sMonID = GetPerfMonByName(sMonName)
 If sMonID <> "0" Then
  Context.LogMessage "   Adding performance monitor " & sMonName
  sSql = "Insert into PivotStatisticalMonitorTypeToDevice (nDeviceID, nStatisticalMonitorTypeId, bEnabled, nDataCollectionInterval, nPollType) " &_
  "Values (" & nDevID & "," & sMonID & ",1,10,0)"
  Set sRS = oDb.Execute(sSql)
 Else
  Context.LogMessage "   Oops, can't find performance monitor " & sMonName
 End If
End If

'Done creating device
Context.LogMessage "Created device " & Name & " the deviceID is " & nDevID
PutDevAttr nDevID, "NetworkName", NetworkName
PutDevAttr nDevID, "Instance", Instance
PutDevAttr nDevID, "Status", Status
PutDevAttr nDevID, "PublicIP", PublicIP
PutDevAttr nDevID, "Serial", Serial
PutDevAttr nDevID, "Model", ProductCode
PutDevAttr nDevID, "Description", ProductDesc

'Start polling the device
SendChangeEvent nDevID
SendPollCommand nDevID

Else
 Context.LogMessage "Did not create " & Name & " because a device with that name already existed with deviceID " & nDevID
End If

End Function

' **************
' * GetDevAttr *
' **************
Function GetDevAttr(nDevID,sAttr)
GetDevAttr = Null
Dim sSql, sRet, sRS
sSql = "Select max(svalue) from DeviceAttribute where nDeviceID = " & nDevID &_
       " and sName = '" & sAttr & "'"
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Right(sRet,1) = vbCr Or Right(sRet,1) = vbLf Then
      sRet = Left(sRet,Len(sRet)-1)
   End If
   If Len(sRet) > 0 Then
      GetDevAttr = Trim(sRet)
   End If
End If
End Function

' ****************
' * GetDevAttrID *
' ****************
Function GetDevAttrID(nDevID,sAttr)
GetDevAttrID = 0
Dim sSql, sRet, sRS
sSql = "Select nDeviceAttributeID from DeviceAttribute where nDeviceID = " & nDevID &_
       " and sName = '" & sAttr & "'"
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Len(sRet) > 0 Then
      If Right(sRet,1) = vbCr Then
         sRet = Left(sRet,Len(sRet)-1)
      End If
   End If
   If IsNumeric(sRet) Then
      GetDevAttrID = CInt(sRet)
   End If
End If
End Function

' **************
' * PutDevAttr *
' **************
Function PutDevAttr(nDevID,sAttr,sVal)
PutDevAttr = Null
Dim sPrev
Dim sSql, sRet, sRS
sPrev = GetDevAttr(nDevID,sAttr)
If GetDevAttrID(nDevID,sAttr) = 0 Then
   ' Need to Insert
   sSql = "Insert into DeviceAttribute(nDeviceID,sName,sValue) " &_
          "Values (" & nDevID & ",'" & sAttr & "','" & sVal & "')"
   Set sRS = oDb.Execute(sSql)
Else
   If sPrev = sVal Then
      ' No Need to do anything
   Else
      ' Need to update
      sSql = "Update DeviceAttribute set sValue = '" & sVal &_
             "' Where nDeviceID = " & nDevID &_
                " And sName = '" & sAttr & "'"
      Set sRS = oDb.Execute(sSql)
   End If
End If
PutDevAttr = 1
End Function

' ****************
' * GetDevByName *
' ****************
Function GetDevByName(sDisplayName)
GetDevByName = "0"
Dim sSql, sRet, sRS
sSql = "Select max(nDeviceID) from Device where sDisplayName = '" & QuoteChars(sDisplayName) & "' And bRemoved = 0"
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
 sRet = Trim(sRS.GetString)
 If Right(sRet,1) = vbCr Or Right(sRet,1) = vbLf Then
  sRet = Left(sRet,Len(sRet)-1)
 End If
 If Len(sRet) > 0 Then
  GetDevByName = Trim(sRet)
 End If
End If
End Function

' ******************
' * GetCreateGroup *
' ******************
Function GetCreateGroup(sName)
Dim sSql, sRS
Dim nID
nID = 0
sSql = "Select Max(nDeviceGroupID) As NID" &_
       " from DeviceGroup where sGroupName = " & StringIfy(sName)
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   nID = sRS("nID")
End If
If nID > 0 Then
 'Do Nothing
Else 'Create group
 sSql = "Insert into DeviceGroup (nParentGroupID, sGroupName, nMonitorStateID, sStatus, sNote, bDynamicGroup, sFilter, sFilterDesign) " &_
   " Values (0," & StringIfy(sName) & ",0,NULL,'Auto-added group',0,NULL,NULL)"
   Set sRS = oDb.Execute(sSql)
   sSql = "Select Max(nDeviceGroupID) As NID" &_
          " from DeviceGroup where sGroupName = " & StringIfy(sName)
   Set sRS = oDb.Execute(sSql)
   If Not sRS.EOF And Not sRS.BOF Then
      nID = CDbl(sRS("nID"))
   End If
End If
GetCreateGroup = nID
End Function

' *******************
' * SendChangeEvent *
' *******************
Sub SendChangeEvent(nDeviceID)
'Variables for device change event
Const DCT_MODIFIED = 2
Const DCIT_DEVICE  = 1
oEvent.SendChangeEvent DCT_MODIFIED, nDeviceID, DCIT_DEVICE
Context.NotifyProgress nDeviceID & " was sent a change event."
End Sub

' *******************
' * SendPollCommand *
' *******************
Sub SendPollCommand(nDeviceID)
'Variables for device change event
Const DCT_MODIFIED = 2
Const DCIT_DEVICE  = 1
oEvent.SendPollCommand nDeviceID
Context.NotifyProgress nDeviceID & " was sent a poll command."
End Sub

' **************
' * GetDevType *
' **************
Function GetDevType(sDeviceType)
Dim sSql, sRS
Dim nDevTypeID
nDevTypeID = 0
sSql = "Select nDeviceTypeID, nDeviceMenuSetID, nDeviceWebMenuSetID " &_
       " from DeviceType where sDisplayName = " & StringIfy(sDeviceType)
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
 nDevTypeID = sRS("nDeviceTypeID")
Else
 If InStr(1, sDeviceType, "AP", 1) <> 0 Then
  sSql = "insert into DeviceType (nDeviceTypeCategoryID, sDisplayName, sMapViewIcon, sOverlayText, nDeviceMenuSetID, nAddressType)" & _
  "Values (0, '" & sDeviceType & "', 'WirelessAccessPoint.png', '" & sDeviceType & "', 49, 1)"
  Context.LogMessage sSql
   Set sRS = oDb.Execute(sSql)
   sSql = "Select nDeviceTypeID, nDeviceMenuSetID, nDeviceWebMenuSetID " &_
        " from DeviceType where sDisplayName = " & StringIfy(sDeviceType)
   Set sRS = oDb.Execute(sSql)
   If Not sRS.EOF And Not sRS.BOF Then
    nDevTypeID = sRS("nDeviceTypeID")
   End If
 Else
  If InStr(1, sDeviceType, "Switch", 1) <> 0 Then
  sSql = "insert into DeviceType (nDeviceTypeCategoryID, sDisplayName, sMapViewIcon, sOverlayText, nDeviceMenuSetID, nAddressType)" & _
  "Values (0, '" & sDeviceType & "', 'Switch.png', '" & sDeviceType & "', 4, 1)"
   Set sRS = oDb.Execute(sSql)
   sSql = "Select nDeviceTypeID, nDeviceMenuSetID, nDeviceWebMenuSetID " &_
        " from DeviceType where sDisplayName = " & StringIfy(sDeviceType)
   Set sRS = oDb.Execute(sSql)
   If Not sRS.EOF And Not sRS.BOF Then
    nDevTypeID = sRS("nDeviceTypeID")
   End If
  End If
 End If
End If
GetDevType = nDevTypeID
End Function

' *************
' * StringIfy *
' *************
Function StringIfy (str)
If IsNull(str) Then
   StringIfy = "NULL"
Else
   StringIfy = "'" & QuoteChars(str) & "'"
End If
End Function 

' **********
' * NumIfy *
' **********
Function NumIfy (num)
If IsNull(num) Then
   NumIfy = "NULL"
Else
   NumIfy = num
End If
End Function
'
' **************
' * QuoteChars *
' **************
Function QuoteChars (ByVal szLine)
Dim rc

If Not IsNull(szLine) And Len(szLine) > 0 then
   rc = InStr(szLine,"'")
   Do While Not IsNull(rc) And rc > 0
      szLine = Left(szLine,rc) & "'" & Right(szLine,Len(szLine)-rc)
      rc = InStr(rc+2,szLine,"'")
   Loop
End If
QuoteChars = szLine
End Function

' *******************
' * GetActMonByName *
' *******************
Function GetActMonByName (sName)
GetActMonByName = "0"
Dim sSql, sRet, sRS
sSql = "Select max(nActiveMonitorTypeID) from ActiveMonitorType where sMonitorTypeName = '" & QuoteChars(sName) & "'"
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Right(sRet,1) = vbCr Or Right(sRet,1) = vbLf Then
      sRet = Left(sRet,Len(sRet)-1)
   End If
   If Len(sRet) > 0 Then
      GetActMonByName = Trim(sRet)
   End If
End If
End Function

' *************
' * DNSLookup *
' *************
Function DNSLookup(sAlias)
Dim sString, arrIP, arrIPTemp, oShell, oExec
  If len(sAlias) = 0 Then
    DNSLookup = "Failed."
    Exit Function
  End If

  Const OpenAsDefault = -2
  Const FailIfNotExist = 0
  Const ForReading = 1

  Set oShell = CreateObject("WScript.Shell")
  Set oExec = oShell.Exec("%comspec% /c nslookup " & sAlias)  
  Do While Not oExec.Stdout.atEndOfStream
   sString = sString +  oExec.StdOut.ReadAll() & vbCrLf
  Loop
 
  arrIP = Split(sString, "Address:")

  If UBound(arrIP) < 2 Then
    DNSLookup = "Failed."
  Else
    arrIPTemp = Split(arrIP(2), Chr(13))
    DNSLookup = trim(arrIPTemp(0))
  End If
  Set oShell = Nothing
End Function

' *********************
' * GetActionPolicyID *
' *********************
Function GetActionPolicyID(sName)
GetActionPolicyID = 0
Dim sSql, sRet, sRS
sSql = "Select max(nActionPolicyID) from ActionPolicy where sPolicyName = '" & QuoteChars(sName) & "'"
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Right(sRet,1) = vbCr Or Right(sRet,1) = vbLf Then
      sRet = Left(sRet,Len(sRet)-1)
   End If
   If Len(sRet) > 0 Then
      GetActionPolicyID = Trim(sRet)
   End If
End If
End Function

' ********************
' * GetNetIntByString*
' ********************
Function GetNetIntByString(nDevID, sNetworkAddress)
GetNetIntByString = "0"
Dim sSql, sRet, sRS
sSql = "select max(nNetworkInterfaceID) from networkinterface where sNetworkAddress = '" & sNetworkAddress & "' and nDeviceID = " & nDevID
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Right(sRet,1) = vbCr Or Right(sRet,1) = vbLf Then
      sRet = Left(sRet,Len(sRet)-1)
   End If
   If Len(sRet) > 0 Then
      GetNetIntByString = Trim(sRet)
   End If
End If
End Function

' ********************
' * GetPerfMonByName *
' ********************
Function GetPerfMonByName (sName)
GetPerfMonByName = "0"
Dim sSql, sRet, sRS
sSql = "Select max(nStatisticalMonitorTypeID) from StatisticalMonitorType where sStatisticalMonitorTypeName = '" & QuoteChars(sName) & "'"
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Right(sRet,1) = vbCr Or Right(sRet,1) = vbLf Then
      sRet = Left(sRet,Len(sRet)-1)
   End If
   If Len(sRet) > 0 Then
      GetPerfMonByName = Trim(sRet)
   End If
End If
End Function

' ****************************************************************************
' * Class clContext ** Class clContext ** Class clContext ** Class clContext *
' ****************************************************************************
Class clContext
		Private internal_StartTime
		Private oDb
		Private bGotDB
		
		Public Function LogMessage(szMsg)
		   Debug.WriteLine szMsg
           wsh.echo szMsg		   
		   LogMessage = 1
		End Function
		
		Public Function NotifyProgress(szMsg)
		   Debug.WriteLine szMsg
           wsh.echo szMsg		   
		   NotifyProgress = 1
		End Function
		
		Public Property Get GetDB
		   If bGotDB = 1 Then
		      GetDB = oDb
		   Else
			  Set oDb = CreateObject("ADODB.Connection")
              oDb.Open "dsn=WhatsUp;uid=sa;pwd=WhatsUp_Gold;"
              'If oDb.Errors.Count <> 0 Then
                 'Dim i
                 'For i = 0 to oDb.errors.count -1
                    'LogMessage "DB Connnect Msg: " + CStr(oDb.Errors.Item(i).NativeError) + " - " + oDb.Errors.Item(i).Description
                 'Next
              'End If
		      Set GetDB = oDb
		      bGotDB = 1
		   End If
		End Property
End Class
