'Instructions: To run this script follows these steps:
'-Add your SNMP credential for Meraki to WhatsUp Gold.
'-Change sSNMPCredName variable under the Configuration section to match the name of this credential
'-Adjust sPathToFile as necessary, ensure you have permissions to write files there
'-Near the bottom of the script, adjust the uid and pwd fields on oDb.Open "dsn=WhatsUp;uid=sa;pwd=WhatsUp_Gold;". These are login credentials to your WUG database
'-Save all your changes, and then run the script from a x86 cmd prompt window like so...
'	-Start > Run > C:\windows\syswow64\cmd.exe [ignore is x86 OS, just open a normal command prompt]
'	-cscript C:\pathToThisScript\gatherdevices.vbs

Option Explicit
Dim Context : Set Context = New clContext ' This line is for running outside of WUG

'****Configuration
Dim sSNMPCredName : sSNMPCredName = "DD Meraki" 'Name of the SNMP credential within WUG to use
Dim sPathToFile : sPathToFile = "C:\wugscripts\meraki_devices.csv" 'Path to the output CSV file
'****End Configuration

'Create objects
Dim oXMLDocument : Set oXMLDocument = CreateObject("Msxml2.DOMDocument.6.0")
Dim oSnmp : Set oSnmp = CreateObject("CoreAsp.SnmpRqst")
Dim oDB : Set oDB = Context.GetDB
Dim oFS : Set oFS = CreateObject("Scripting.FileSystemObject")
Dim oTextFile : Set oTextFile = oFS.CreateTextFile(sPathToFile)
'SNMP OIDs
Dim devTableInit : devTableInit = "1.3.6.1.4.1.29671.1.1.4.1.1"
Dim devTable : devTable = "1.3.6.1.4.1.29671.1.1.4.1.1"
Dim devName : devName = "1.3.6.1.4.1.29671.1.1.4.1.2."
Dim devNetworkName : devNetworkName = "1.3.6.1.4.1.29671.1.1.4.1.11."
Dim devStatus : devStatus = "1.3.6.1.4.1.29671.1.1.4.1.3."
Dim devPublicIP : devPublicIP = "1.3.6.1.4.1.29671.1.1.4.1.7."
Dim devSerial : devSerial = "1.3.6.1.4.1.29671.1.1.4.1.8."
Dim devProductCode : devProductCode = "1.3.6.1.4.1.29671.1.1.4.1.9."
Dim devProductDescription : devProductDescription = "1.3.6.1.4.1.29671.1.1.4.1.10."
'Constants
Const sMerakiSNMP = "snmp.meraki.com" 'Hostname to Meraki SNMP
Const nPort = 16100 'SNMP Port to Use
Const nTimeoutValue = 3000 'SNMP Timeout value in ms
Const nRetries = 1 'SNMP retry value
'Variable declaration
Dim bFail : bFail = 0
Dim bDown : bDown = 0
Dim bSNMPResult : bSNMPResult = 0
Dim bCount : bCount = 0
Dim bTryAgain : bTryAgain = 0
Dim sErrorMsg : sErrorMsg = ""
'Get the credential ID
Dim sSNMPCred : sSNMPCred = GetCredID(oDB, sSNMPCredName)
'Write the header line to CSV
oTextFile.WriteLine "Name,NetworkName,Instance,Status,PublicIP,Serial,ProductCode,ProductDesc"

Dim rc, sInstance, sNewLine, sPreviousLine

InitializeSNMP
'Loop for values
If bFail = 0 And bSNMPResult = 0 Then
 Do
 bCount = bCount + 1
 'Context.LogMessage bCount
  If bTryAgain = 0 Then
   Set rc = oSnmp.GetNext(devTable)
   If rc.Failed Then
    sErrorMsg = rc.GetErrorMsg
    Context.LogMessage "Failed to GetNext SNMP value...will try again!"
    bTryAgain = 1
   Else
    devTable = rc.GetOID
    If InStr(devTable, "1.3.6.1.4.1.29671.1.1.4.1.1.") <> 0 Then
     sInstance = rc.GetOIDInstance
    Else
     Exit Do
    End If
   End If
  End If
  If bTryAgain = 1 Then
   InitializeSNMP
  End If
  If bTryAgain = 1 Or bTryAgain = 0 Then
   'Get SNMP values
   Dim sResult, sDisplayName, sName, sNetworkName
   'Clear the variables
   sResult = ""
   oVarBinds = ""
   Dim Name : Name = ""
   Dim NetworkName : NetworkName = ""
   Dim MAC : MAC = ""
   Dim ContactAt : ContactAt = ""
   Dim Status : Status = ""
   Dim PublicIP : PublicIP = ""
   Dim Serial : Serial = ""
   Dim ProductCode : ProductCode = ""
   Dim ProductDesc : ProductDesc = ""
   Dim oSnmpResponse, sXML, oVarBinds
   '***Get the SNMP values in a single packet
   'Context.LogMessage "Trying to get info from:" & sInstance
   Set oSnmpResponse = oSnmp.GetMultiple(devName & sInstance & "," & devNetworkName & sInstance & "," & devStatus & sInstance & "," & devPublicIP & sInstance & "," & devSerial & sInstance & "," & devProductCode & sInstance & "," & devProductDescription & sInstance)
   If oSnmpResponse.Failed Then
    bTryAgain = 1
    Context.LogMessage "Failed...will reconnect to SNMP and try again!"
   Else
    'On Error Resume Next
    sNewLine = ""
    sXML = oSnmpResponse.GetPayload
    oXMLDocument.async = False
    oXMLDocument.loadXML(sXML)
    Set oVarBinds = oXMLDocument.getElementsByTagName("SnmpVarBind")
    Name = oVarBinds(0).getAttribute("sValue")
    NetworkName = oVarBinds(1).getAttribute("sValue")
    Status = oVarBinds(2).getAttribute("sValue")
    PublicIP = oVarBinds(3).getAttribute("sValue")
    Serial = oVarBinds(4).getAttribute("sValue")
    ProductCode = oVarBinds(5).getAttribute("sValue")
    ProductDesc = oVarBinds(6).getAttribute("sValue")
    If Status = 1 Then
     Status = "Online"
    Else
     Status = "Offline"
    End If
    sNewLine = Name & "," & NetworkName &  ","  & sInstance & "," & Status & "," & PublicIP & "," & Serial & "," & ProductCode & "," & ProductDesc
    If sNewLine <> sPreviousLine Then 
     'Write to CSV
     oTextFile.WriteLine Name & "," & NetworkName &  ","  & sInstance & "," & Status & "," & PublicIP & "," & Serial & "," & ProductCode & "," & ProductDesc
     sPreviousLine = Name & "," & NetworkName &  ","  & sInstance & "," & Status & "," & PublicIP & "," & Serial & "," & ProductCode & "," & ProductDesc
     'Write a log message
     Context.LogMessage Name & "," & NetworkName &  ","  & sInstance & "," & Status & "," & PublicIP & "," & Serial & "," & ProductCode & "," & ProductDesc
    Else
     Context.LogMessage "Already wrote that line, skipping to avoid duplicate..."
    End If
   End If
  End If
 Loop Until InStr(devTable, "1.3.6.1.4.1.29671.1.1.4.1.1") = 0
End If

' ****************
' *  GetSNMPVal  *
' ****************
Function GetSNMPVal(sOID, ByRef bFail, ByRef sErrorMsg)
GetSNMPVal = 0
Set rc = oSnmp.Get(sOID)
If rc.Failed Then
 sErrorMsg = rc.GetErrorMsg
 bFail = bFail + 1
Else
'Context.LogMessage rc.GetValue
GetSNMPVal = rc.GetValue
End If
End Function

' *******************
' * GetCredentialID *
' *******************
Function GetCredID(oDB, sSNMPCred)
GetCredID = 0
Dim sSql, sRet, sRS
   sSql = "Select max(nCredentialTypeID) AS nID from CredentialType " & _
          " Where nCredentialType in (1,2,4) " & _
          " and sDisplayName = " & StringIfy(sSNMPCred)
   Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Len(sRet) > 0 Then
      If Right(sRet, 1) = vbCr Then
         sRet = Left(sRet, Len(sRet) - 1)
      End If
   End If
   If IsNumeric(sRet) Then
      GetCredID = CInt(sRet)
   End If
End If
End Function

' *************
' * StringIfy *
' *************
Function StringIfy(str)
If IsNull(str) Then
   StringIfy = "NULL"
Else
   StringIfy = "'" & QuoteChars(str) & "'"
End If
End Function

' **********
' * NumIfy *
' **********
Function NumIfy(num)
If IsNull(num) Then
   NumIfy = "NULL"
Else
   NumIfy = num
End If
End Function

' **************
' * QuoteChars *
' **************
Function QuoteChars(ByVal szLine)
Dim rc

If Not IsNull(szLine) And Len(szLine) > 0 Then
   rc = InStr(szLine, "'")
   Do While Not IsNull(rc) And rc > 0
      szLine = Left(szLine, rc) & "'" & Right(szLine, Len(szLine) - rc)
      rc = InStr(rc + 2, szLine, "'")
   Loop
End If
QuoteChars = szLine
End Function

Sub InitializeSNMP()
'Initialize SNMP
Dim timeout, port, NumRetries
Set rc = Nothing
Set rc = oSnmp.Initialize2(sMerakiSNMP, sSNMPCred)
Set timeout = oSnmp.SetTimeoutMs(nTimeoutValue)
Set NumRetries = oSnmp.SetNumRetries(nRetries)
Set port = oSnmp.SetPort(nPort)
If rc.Failed Then
 sErrorMsg = rc.GetErrorMsg
 bSNMPResult = 1
Else
 If bTryAgain = 0 Then
  Context.LogMessage "Successfully connected to SNMP..."
 ElseIf bTryAgain = 1 Then
  Context.LogMessage "Successfully reconnected to SNMP..."
  bTryAgain = 0
 End If
End If
End Sub

' ****************************************************************************
' * Class clContext ** Class clContext ** Class clContext ** Class clContext *
' ****************************************************************************
Class clContext
		Private internal_StartTime
		Private oDb
		Private bGotDB
		
		Public Function LogMessage(szMsg)
		   Debug.WriteLine szMsg
           wsh.echo szMsg		   
		   LogMessage = 1
		End Function
		
		Public Function NotifyProgress(szMsg)
		   Debug.WriteLine szMsg
           wsh.echo szMsg		   
		   NotifyProgress = 1
		End Function
		
		Public Property Get GetDB
		   If bGotDB = 1 Then
		      GetDB = oDb
		   Else
			  Set oDb = CreateObject("ADODB.Connection")
              oDb.Open "dsn=WhatsUp;uid=sa;pwd=WhatsUp_Gold;"
              'If oDb.Errors.Count <> 0 Then
                 'Dim i
                 'For i = 0 to oDb.errors.count -1
                    'LogMessage "DB Connnect Msg: " + CStr(oDb.Errors.Item(i).NativeError) + " - " + oDb.Errors.Item(i).Description
                 'Next
              'End If
		      Set GetDB = oDb
		      bGotDB = 1
		   End If
		End Property
End Class
