'NAME: Get SNMP value from instance [the OID value under credentials] assigned to device
'AUTHOR: Jason Alberino, Ipswitch, Inc.
'DATE: 04/20/2016
Option Explicit
'*****Configuration*****
Dim sSNMPCredName : sSNMPCredName = "DD Meraki" 'Name of the credential to lookup
Dim nSNMPTimeout : nSNMPTimeout = 3000 'Set ms for SNMP timeouts
Dim nSNMPRetry : nSNMPRetry = 6 'Set the number of SNMP retries
Dim nPort : nPort = 16100 'snmp port
Dim sHostname : sHostname = "snmp.meraki.com" 'snmp hostname
'***End Configuration***

'Set base variables
Dim sErrorMsg : sErrorMsg = ""
Dim bFail : bFail = 0
Dim bSNMPResult : bSNMPResult = 0
'Get Device details
Dim nDeviceID : nDeviceID = Context.GetProperty("DeviceID")
Dim sAddress : sAddress = Context.GetProperty("Address")
'Create SNMP object
Dim oSnmp : Set oSnmp = CreateObject("CoreAsp.SnmpRqst")
'Open DB connection
Dim oDb : Set oDb = Context.GetDB
'OID to check
Dim devStatus : devStatus = "1.3.6.1.4.1.29671.1.1.4.1.3."
'Instance of device
Dim sInstance : sInstance = GetDeviceOID(nDeviceID)
'Get credential ID
Dim sSNMPCred : sSNMPCred = GetCredID(oDb, sSNMPCredName)

'Test SNMP
TestSNMP
If bSNMPResult = 0 Then
 Dim bStatus : bStatus = GetSNMPVal(devStatus & sInstance)
 If bFail = 1 Then
  bStatus = GetSNMPVal(devStatus & sInstance)
 End If
  SetResults bFail, sErrorMsg, bSNMPResult, bStatus
Else
 SetResults bFail, sErrorMsg, bSNMPResult, bStatus
End If

' ****************
' *  GetSNMPVal  *
' ****************
Function GetSNMPVal(sOID)
Dim rc
GetSNMPVal = 0
bFail = 0
Context.LogMessage sOID
Set rc = oSnmp.Get(sOID)
If rc.Failed Then
 sErrorMsg = rc.GetErrorMsg
 bFail = bFail + 1
 InitializeSNMP
Else
 Context.LogMessage rc.GetValue
 GetSNMPVal = rc.GetValue
End If
End Function

' **************
' *  TestSNMP  *
' **************
Sub TestSNMP()
Dim rc, sOID, timeout, NumRetries, port
'Initialize SNMP
Set rc = oSnmp.Initialize2(sHostname, sSNMPCred)
Set timeout = oSnmp.SetTimeoutMs(nSNMPTimeout)
Set NumRetries = oSnmp.SetNumRetries(nSNMPRetry)
Set port = oSnmp.SetPort(nPort)
If rc.Failed Then
 sErrorMsg = rc.GetErrorMsg
 bSNMPResult = 1
 bFail = 1
Else
 Set rc = oSnmp.GetNext("1.3.6.1.4.1.29671.1.1.4.1.3")
 If rc.Failed Then
  sErrorMsg = rc.GetErrorMsg
  bSNMPResult =  1
  bFail = 1
  Context.LogMessage "SNMP initialization failed:" & sErrorMsg
 Else
  sOID = rc.GetValue
  Context.LogMessage "SNMP test successful!" & vbCrLf
 End If
End If
End Sub

Sub InitializeSNMP()
'Initialize SNMP
Dim rc, timeout, port, NumRetries
Set rc = Nothing
Set rc = oSnmp.Initialize2(sMerakiSNMP, sSNMPCred)
Set timeout = oSnmp.SetTimeoutMs(nTimeoutValue)
Set NumRetries = oSnmp.SetNumRetries(nRetries)
Set port = oSnmp.SetPort(nPort)
If rc.Failed Then
 sErrorMsg = rc.GetErrorMsg
 bSNMPResult = 1
Else
 If bTryAgain = 0 Then
  Context.LogMessage "Successfully connected to SNMP..."
 ElseIf bTryAgain = 1 Then
  Context.LogMessage "Successfully reconnected to SNMP..."
  bTryAgain = 0
 End If
End If
End Sub

Sub SetResults(bFail, sErrorMsg, bSNMPResult, bDown)
'Set result based on errors
Dim bResultSet : bResultSet = 0
If bSNMPResult = 1 Then
 bResultSet = 1
 Context.SetResult 1, "Error: " & sErrorMsg
Else
 If bFail = 1 Then
  If sErrorMsg = "No Such object" Then		
   bResultSet = 1
   Context.LogMessage "Down!"
   Context.SetResult 1, "Cannot find the SNMP object"
  ElseIf sErrorMsg = "Timeout" Then
   bResultSet = 1
   Context.LogMessage "Down!"
   Context.SetResult 1, "SNMP intilization timed out"
  ElseIf sErrorMsg <> "No Such object" And sErrorMsg <> "Timeout" Then
   bResultSet = 1
   Context.LogMessage "Down!"
   Context.SetResult 1, "There was a SNMP error" & vbCrLf & "Error: " & sErrorMsg
  End If
 Else
 End If
End If

'Set Up or Down
If bResultSet = 0 Then
 If bDown = 0 Then
  Context.LogMessage "Down!"
  Context.SetResult 1, "Status is offline."
 Else
  Context.LogMessage "UP!"
  Context.SetResult 0, "Status is online."
 End If
End If
End Sub

' ****************
' * GetDeviceOID *
' ****************
Function GetDeviceOID(nDeviceID)
GetDeviceOID = 0
Dim sRet, sRS, sSql
sSql = "select sSnmpOID from Device where nDeviceID = " & nDeviceID
Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Right(sRet, 1) = vbCr Or Right(sRet, 1) = vbLf Then
      sRet = Left(sRet, Len(sRet) - 1)
   End If
   If Len(sRet) > 0 Then
      GetDeviceOID = Trim(sRet)
   End If
End If
End Function

' *******************
' * GetCredentialID *
' *******************
Function GetCredID(oDB, sSNMPCred)
GetCredID = 0
Dim sSql, sRet, sRS
   sSql = "Select max(nCredentialTypeID) AS nID from CredentialType " & _
          " Where nCredentialType in (1,2,4) " & _
          " and sDisplayName = " & StringIfy(sSNMPCred)
   Set sRS = oDb.Execute(sSql)
If Not sRS.EOF And Not sRS.BOF Then
   sRet = Trim(sRS.GetString)
   If Len(sRet) > 0 Then
      If Right(sRet, 1) = vbCr Then
         sRet = Left(sRet, Len(sRet) - 1)
      End If
   End If
   If IsNumeric(sRet) Then
      GetCredID = CInt(sRet)
   End If
End If
End Function

' *************
' * StringIfy *
' *************
Function StringIfy (str)
If IsNull(str) Then
   StringIfy = "NULL"
Else
   StringIfy = "'" & QuoteChars(str) & "'"
End If
End Function 

' **************
' * QuoteChars *
' **************
Function QuoteChars (ByVal szLine)
Dim rc

If Not IsNull(szLine) And Len(szLine) > 0 Then
   rc = InStr(szLine,"'")
   Do While Not IsNull(rc) And rc > 0
      szLine = Left(szLine,rc) & "'" & Right(szLine,Len(szLine)-rc)
      rc = InStr(rc+2,szLine,"'")
   Loop
End If
QuoteChars = szLine
End Function
